import pytz
from datetime import datetime, timezone
from sqlalchemy import DateTime as SdateTime
from sqlalchemy.types import TypeDecorator
from uuid import uuid4
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.postgresql import UUID, VARCHAR, TIMESTAMP, SMALLINT, JSONB, TEXT, BOOLEAN, BIGINT, INTEGER, FLOAT
from sqlalchemy import Column, create_engine, func, ForeignKey


# Mexico city time zone
mx_cty_tz = pytz.timezone('America/Mexico_City')

# Function to get timezone aware current time


def now(): return datetime.utcnow().replace(
    tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)


Base = declarative_base()


class DateTime(TypeDecorator):
    impl = SdateTime

    def process_bind_param(self, value, engine):
        if value is None:
            return value
        return value.replace(tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)

    def process_result_value(self, value, engine):
        return value

class AlembicVersion(Base):
    __tablename__ = "alembic_version"
    version_num = Column(VARCHAR(32), primary_key=True)

class Workers(Base):
    __tablename__ = "workers"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    persona = Column(VARCHAR(12))
    name = Column(VARCHAR(50))
    last_name = Column(VARCHAR(50))
    email = Column(VARCHAR(150))
    username = Column(VARCHAR(30))
    status = Column(SMALLINT)
    logged_at = Column(TIMESTAMP)
    session_deadline = Column(TIMESTAMP)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)
    session_id = Column(VARCHAR(50))


class WorkerHistory(Base):
    __tablename__ = "worker_history"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    worker_uuid = Column(UUID(as_uuid=True), ForeignKey(Workers.uuid), nullable=False)
    status_code = Column(VARCHAR(20))
    status = Column(VARCHAR(20))
    reason = Column(TEXT)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)


class Tasks(Base):
    __tablename__ = "tasks"

    uuid = Column(VARCHAR(50), primary_key=True)
    worker_uuid = Column(UUID(as_uuid=True), ForeignKey(Workers.uuid), nullable=True)
    name = Column(VARCHAR(150))
    data = Column(JSONB)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    priority = Column(SMALLINT)
    deadline = Column(TIMESTAMP)
    requested_at = Column(TIMESTAMP)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)


class TaskHistory(Base):
    __tablename__ = "task_history"

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    task_uuid = Column(VARCHAR(50), ForeignKey(Tasks.uuid), nullable=False)
    worker_uuid = Column(UUID(as_uuid=True), ForeignKey(Workers.uuid), nullable=False)
    data = Column(JSONB)
    reason = Column(TEXT)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)


class Formularios(Base):
    __tablename__ = "formularios"
    id = Column(SMALLINT, primary_key=True)
    name = Column(VARCHAR(30))


class Stages(Base):
    __tablename__ = "stages"
    id = Column(SMALLINT, primary_key=True)
    data = Column(JSONB)
    formulario_id = Column(SMALLINT, ForeignKey(Formularios.id), nullable=False)
    name = Column(VARCHAR(30))
    order = Column(SMALLINT)


class News(Base):
    __tablename__ = "news"

    new_id = Column(BIGINT, autoincrement=True, primary_key=True)
    title = Column(VARCHAR(128), default='', nullable=False)
    resume = Column(VARCHAR(1024), default='', nullable=False)
    date_begin_publish = Column(DateTime, default=now, nullable=True)
    date_end_publish = Column(TIMESTAMP)
    active = Column(BOOLEAN, default=True)
    image = Column(VARCHAR(2048))

class Parameters(Base):
    __tablename__ = "parameters"
    id = Column(BIGINT, autoincrement=True, primary_key=True)
    task_identifier = Column(VARCHAR(30))
    parameter_key = Column(VARCHAR(50), nullable=False)
    parameter_value = Column(VARCHAR(200), nullable=False)
    description = Column(VARCHAR(200), nullable=False)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)

class Codes(Base):
    __tablename__ = "codes"
    id = Column(BIGINT, autoincrement=True, primary_key=True)
    task_identifier = Column(VARCHAR(30))
    code_type = Column(VARCHAR(30), nullable=False) 
    code_key = Column(TEXT, nullable=False)
    code_value = Column(TEXT, nullable=False)
    description = Column(VARCHAR(200), nullable=False)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)

class Casbin(Base):
    __tablename__ = "casbin_rule"

    id = Column(INTEGER, autoincrement=True, primary_key=True)
    ptype = Column(VARCHAR(255), nullable=True)
    v0 = Column(VARCHAR(255), nullable=True)
    v1 = Column(VARCHAR(255), nullable=True)
    v2 = Column(VARCHAR(255), nullable=True)
    v3 = Column(VARCHAR(255), nullable=True)
    v4 = Column(VARCHAR(255), nullable=True)
    v5 = Column(VARCHAR(255), nullable=True)

class WorkerScore(Base):
    __tablename__ = "worker_score"

    id = Column(BIGINT, autoincrement=True, primary_key=True)
    worker_uuid = Column(UUID(as_uuid=True), ForeignKey(Workers.uuid), nullable=False)
    task_identifier = Column(VARCHAR(50), nullable=False)
    total_points = Column(FLOAT, nullable=True)
    total_attended_task = Column(BIGINT, nullable=True)
    total_secs = Column(BIGINT, nullable=True)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)